# encoding: utf-8
'''Cache-related utilities.
'''
from types import *

__all__ = ['callable_cache_key']

def callable_cache_key(node):
  '''Calculate key unique enought to be used for caching callables.
  '''
  if not isinstance(node, (MethodType, FunctionType)):
    return hash(node.__call__)^hash(node)
  elif isinstance(node, MethodType):
    return hash(node)^hash(node.im_class)
  return node



# encoding: utf-8
'''
JSON: JavaScript Object Notation

:see: `RFC 4627 <http://tools.ietf.org/html/rfc4627>`__
:requires: `cjson <http://pypi.python.org/pypi/python-cjson>`__ | minjson
'''
from smisk.core import request
from smisk.serialization import serializers, Serializer
try:
  from cjson import encode as json_encode, decode as json_decode,\
                    DecodeError, EncodeError
except ImportError:
  try:
    from minjson import write as json_encode, read as json_decode,\
                        ReadException as DecodeError,\
                        WriteException as EncodeError
  except ImportError:
    json_encode = None
    from warnings import warn
    warn('No JSON implementation available. Install cjson or minjson.')

class JSONSerializer(Serializer):
  '''JSON with JSONP support.
  
  JSONP support through passing the special ``callback`` query string parameter.
  '''
  name = 'JSON: JavaScript Object Notation'
  extensions = ('json',)
  media_types = ('application/json',)
  
  @classmethod
  def serialize(cls, params, charset):
    callback = None
    if request:
      callback = request.get.get('callback', None)
    if callback:
      return (None, '%s(%s);' % (callback, json_encode(params)))
    else:
      return (None, json_encode(params))
  
  @classmethod
  def serialize_error(cls, status, params, charset):
    return (None, json_encode(params))
  
  @classmethod
  def unserialize(cls, file, length=-1, charset=None):
    # return (list args, dict params)
    st = json_decode(file.read(length))
    if isinstance(st, dict):
      return (None, st)
    elif isinstance(st, list):
      return (st, None)
    else:
      return ((st,), None)
  

# Don't register if we did not find a json implementation
if json_encode is not None:
  serializers.register(JSONSerializer)

if __name__ == '__main__':
  s = JSONSerializer.serialize({
    'message': 'Hello worlds',
    'internets': [
      'interesting',
      'lolz',
      42.0,
      {
        'tubes': [1,3,16,18,24],
        'persons': True,
        'me agåain': {
          'message': 'Hello worlds',
          'internets': [
            'interesting',
            'lolz',
            42.0,
            {
              'tubes': [1,3,16,18,24],
              'persons': True
            }
          ]
        }
      }
    ]
  }, None)
  print s


#!/usr/bin/env python
# encoding: utf-8
'''Unit test suite.
'''
import unittest, os

class TestCase(unittest.TestCase):
  def assertContains(self, collection1, collection2):
    '''Test that all items in `collection1` is contained within `collection2`.
    
    The order of the contained items does not matter.
    
    :param collection1: Collection 1
    :type  collection1: collection
    :param collection2: Collection 2
    :type  collection2: collection
    :rtype: None
    '''
    for item in collection1:
      if item not in collection2:
        raise unittest.AssertionError(u'%r !contains %r' % (collection1, collection2))
  

def suite():
  from smisk.util.python import load_modules
  suites = []
  for m in load_modules(os.path.dirname(__file__), deep=True).values():
    try:
      suites.append(m.suite())
    except AttributeError:
      pass
  return unittest.TestSuite(suites)

def test():
  runner = unittest.TextTestRunner()
  return runner.run(suite())

if __name__ == "__main__":
  test()



#!/usr/bin/env python
# encoding: utf-8
import sys, os
from smisk.core import *

class MyApp(Application):
  def __init__(self, *args, **kwargs):
    super(MyApp, self).__init__(*args, **kwargs)
    # Set a very low session TTL for easy demonstration
    self.sessions.ttl = 10
  
  def service(self):
    self.response.headers = ["Content-Type: text/plain"]
    
    # Dump raw input?
    if 'dump' in self.request.get:
      while 1:
        chunk = self.request.input.read(8192)
        self.response.out.write(repr(chunk).strip("'").replace(r'\n', "\\n\n"))
        if len(chunk) < 8192:
          break
      return
    
    # Set a cookie
    if self.request.get.has_key('set_cookie'):
      self.response.set_cookie('a_cookie', self.request.get['set_cookie'], max_age=20)
    
    # Add some session data
    if self.request.get.has_key('set_session'):
      if self.request.get['set_session'] == '':
        self.request.session = None
      else:
        self.request.session = self.request.get['set_session']
    elif self.request.session is None:
      self.request.session = 'mos'
    
    # Print alot of information
    w = self.response.write
    w("self. %s\n" % repr(self))
    w(" request_class:       %s\n" % repr(self.request_class))
    w(" response_class       %s\n" % repr(self.response_class))
    w(" sessions_class: %s\n" % repr(self.sessions_class))
    w(" sessions.       %s\n" % repr(self.sessions))
    w("  name:                %s\n" % repr(self.sessions.name))
    w("  ttl:                 %d\n" % self.sessions.ttl)
    w("\n")
    w("self.request.\n")
    w(" env         %s\n" % repr(self.request.env))
    w(" get         %s\n" % repr(self.request.get))
    w(" post        %s\n" % repr(self.request.post))
    w(" files       %s\n" % repr(self.request.files))
    w(" cookies     %s\n" % repr(self.request.cookies))
    w(" input       %s\n" % repr(self.request.input.read()))
    w(" url         %s\n" % self.request.url)
    w(" session_id: %s\n" % repr(self.request.session_id))
    w(" session:    %s\n" % repr(self.request.session))
    w("\n")
    w("self.response.\n")
    w(" headers     %s\n" % repr(self.response.headers))
  

if __name__ == '__main__':
  from smisk.util.main import main
  main(MyApp)
